/*
 *  $Id: global_ops.c,v 1.3 1993/11/12 11:58:25 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */


#ifndef lint
static char vcid[] = "$Id: global_ops.c,v 1.3 1993/11/12 11:58:25 doss Exp $";
#endif /* lint */

/*D

Global Compute Operations - 

D*/

#include <mpi.h>
#include "coll.h"

void MPI_MAX( invec, inoutvec, len, type )
void *invec, *inoutvec;
int  len;
MPI_Datatype type;
{
  int i;

  switch (type->dte_type) {
  case MPIR_INT: {
    int *a = (int *)inoutvec; int *b = (int *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_MAX(a[i],b[i]);
    break;
  }
  case MPIR_LONG: {
    long *a = (long *)inoutvec; long *b = (long *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_MAX(a[i],b[i]);
    break;
  }
  case MPIR_SHORT: {
    short *a = (short *)inoutvec; short *b = (short *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_MAX(a[i],b[i]);
    break;
  }
  case MPIR_FLOAT: {
    float *a = (float *)inoutvec; float *b = (float *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_MAX(a[i],b[i]);
    break;
  }
  case MPIR_DOUBLE: {
    double *a = (double *)inoutvec; double *b = (double *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_MAX(a[i],b[i]);
    break;
  }
  default:
    break;
  }
}

void MPI_MIN ( invec, inoutvec, len, type )
void *invec, *inoutvec;
int  len;
MPI_Datatype type;
{
  int i;

  switch (type->dte_type) {
  case MPIR_INT: {
    int *a = (int *)inoutvec; int *b = (int *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_MIN(a[i],b[i]);
    break;
  }
  case MPIR_LONG: {
    long *a = (long *)inoutvec; long *b = (long *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_MIN(a[i],b[i]);
    break;
  }
  case MPIR_SHORT: {
    short *a = (short *)inoutvec; short *b = (short *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_MIN(a[i],b[i]);
    break;
  }
  case MPIR_FLOAT: {
    float *a = (float *)inoutvec; float *b = (float *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_MIN(a[i],b[i]);
    break;
  }
  case MPIR_DOUBLE: {
    double *a = (double *)inoutvec; double *b = (double *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_MIN(a[i],b[i]);
    break;
  }
  default:
    break;
  }
}


#ifndef MPIR_SUM
#define MPIR_SUM(a,b) ((a)+(b))
#endif
void MPI_SUM ( invec, inoutvec, len, type )
void *invec, *inoutvec;
int  len;
MPI_Datatype type;
{
  int i;

  switch (type->dte_type) {
  case MPIR_INT: {
    int *a = (int *)inoutvec; int *b = (int *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_SUM(a[i],b[i]);
    break;
  }
  case MPIR_LONG: {
    long *a = (long *)inoutvec; long *b = (long *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_SUM(a[i],b[i]);
    break;
  }
  case MPIR_SHORT: {
    short *a = (short *)inoutvec; short *b = (short *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_SUM(a[i],b[i]);
    break;
  }
  case MPIR_FLOAT: {
    float *a = (float *)inoutvec; float *b = (float *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_SUM(a[i],b[i]);
    break;
  }
  case MPIR_DOUBLE: {
    double *a = (double *)inoutvec; double *b = (double *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_SUM(a[i],b[i]);
    break;
  }
  default:
    break;
  }
}



#define MPIR_PROD(a,b) ((a)*(b))
void MPI_PROD ( invec, inoutvec, len, type )
void *invec, *inoutvec;
int  len;
MPI_Datatype type;
{
  int i;

  switch (type->dte_type) {
  case MPIR_INT: {
    int *a = (int *)inoutvec; int *b = (int *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_PROD(a[i],b[i]);
    break;
  }
  case MPIR_LONG: {
    long *a = (long *)inoutvec; long *b = (long *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_PROD(a[i],b[i]);
    break;
  }
  case MPIR_SHORT: {
    short *a = (short *)inoutvec; short *b = (short *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_PROD(a[i],b[i]);
    break;
  }
  case MPIR_FLOAT: {
    float *a = (float *)inoutvec; float *b = (float *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_PROD(a[i],b[i]);
    break;
  }
  case MPIR_DOUBLE: {
    double *a = (double *)inoutvec; double *b = (double *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_PROD(a[i],b[i]);
    break;
  }
  default:
    break;
  }
}



#ifndef MPIR_LAND
#define MPIR_LAND(a,b) ((a)&&(b))
#endif
void MPI_LAND ( invec, inoutvec, len, type )
void *invec, *inoutvec;
int  len;
MPI_Datatype type;
{
  int i;

  switch (type->dte_type) {
  case MPIR_INT: {
    int *a = (int *)inoutvec; int *b = (int *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LAND(a[i],b[i]);
    break;
  }
  case MPIR_LONG: {
    long *a = (long *)inoutvec; long *b = (long *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LAND(a[i],b[i]);
    break;
  }
  case MPIR_SHORT: {
    short *a = (short *)inoutvec; short *b = (short *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LAND(a[i],b[i]);
    break;
  }
  case MPIR_FLOAT: {
    float *a = (float *)inoutvec; float *b = (float *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LAND(a[i],b[i]);
    break;
  }
  case MPIR_DOUBLE: {
    double *a = (double *)inoutvec; double *b = (double *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LAND(a[i],b[i]);
    break;
  }
  default:
    break;
  }
}



#ifndef MPIR_BAND
#define MPIR_BAND(a,b) ((a)&(b))
#endif
void MPI_BAND ( invec, inoutvec, len, type )
void *invec, *inoutvec;
int  len;
MPI_Datatype type;
{
  int i;

  switch (type->dte_type) {
  case MPIR_INT: {
    int *a = (int *)inoutvec; int *b = (int *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_BAND(a[i],b[i]);
    break;
  }
  case MPIR_LONG: {
    long *a = (long *)inoutvec; long *b = (long *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_BAND(a[i],b[i]);
    break;
  }
  case MPIR_SHORT: {
    short *a = (short *)inoutvec; short *b = (short *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_BAND(a[i],b[i]);
    break;
  }
  default:
    break;
  }
}



#ifndef MPIR_LOR
#define MPIR_LOR(a,b) ((a)||(b))
#endif
void MPI_LOR ( invec, inoutvec, len, type )
void *invec, *inoutvec;
int  len;
MPI_Datatype type;
{
  int i;

  switch (type->dte_type) {
  case MPIR_INT: {
    int *a = (int *)inoutvec; int *b = (int *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LOR(a[i],b[i]);
    break;
  }
  case MPIR_LONG: {
    long *a = (long *)inoutvec; long *b = (long *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LOR(a[i],b[i]);
    break;
  }
  case MPIR_SHORT: {
    short *a = (short *)inoutvec; short *b = (short *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LOR(a[i],b[i]);
    break;
  }
  case MPIR_FLOAT: {
    float *a = (float *)inoutvec; float *b = (float *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LOR(a[i],b[i]);
    break;
  }
  case MPIR_DOUBLE: {
    double *a = (double *)inoutvec; double *b = (double *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LOR(a[i],b[i]);
    break;
  }
  default:
    break;
  }
}


#ifndef MPIR_BOR
#define MPIR_BOR(a,b) ((a)|(b))
#endif
void MPI_BOR ( invec, inoutvec, len, type )
void *invec, *inoutvec;
int  len;
MPI_Datatype type;
{
  int i;

  switch (type->dte_type) {
  case MPIR_INT: {
    int *a = (int *)inoutvec; int *b = (int *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_BOR(a[i],b[i]);
    break;
  }
  case MPIR_LONG: {
    long *a = (long *)inoutvec; long *b = (long *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_BOR(a[i],b[i]);
    break;
  }
  case MPIR_SHORT: {
    short *a = (short *)inoutvec; short *b = (short *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_BOR(a[i],b[i]);
    break;
  }
  default:
    break;
  }
}



#ifndef MPIR_LXOR
#define MPIR_LXOR(a,b) (((a)&&(!b))||((!a)&&(b)))
#endif
void MPI_LXOR ( invec, inoutvec, len, type )
void *invec, *inoutvec;
int  len;
MPI_Datatype type;
{
  int i;

  switch (type->dte_type) {
  case MPIR_INT: {
    int *a = (int *)inoutvec; int *b = (int *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LXOR(a[i],b[i]);
    break;
  }
  case MPIR_LONG: {
    long *a = (long *)inoutvec; long *b = (long *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LXOR(a[i],b[i]);
    break;
  }
  case MPIR_SHORT: {
    short *a = (short *)inoutvec; short *b = (short *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LXOR(a[i],b[i]);
    break;
  }
  case MPIR_FLOAT: {
    float *a = (float *)inoutvec; float *b = (float *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LXOR(a[i],b[i]);
    break;
  }
  case MPIR_DOUBLE: {
    double *a = (double *)inoutvec; double *b = (double *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_LXOR(a[i],b[i]);
    break;
  }
  default:
    break;
  }
}



#ifndef MPIR_BXOR
#define MPIR_BXOR(a,b) ((a)^(b))
#endif
void MPI_BXOR ( invec, inoutvec, len, type )
void *invec, *inoutvec;
int  len;
MPI_Datatype type;
{
  int i;

  switch (type->dte_type) {
  case MPIR_INT: {
    int *a = (int *)inoutvec; int *b = (int *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_BXOR(a[i],b[i]);
    break;
  }
  case MPIR_LONG: {
    long *a = (long *)inoutvec; long *b = (long *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_BXOR(a[i],b[i]);
    break;
  }
  case MPIR_SHORT: {
    short *a = (short *)inoutvec; short *b = (short *)invec;
    for ( i=0; i<len; i++ ) 
      a[i] = MPIR_BXOR(a[i],b[i]);
    break;
  }
  default:
    break;
  }
}


