/*
 *  $Id: u_scan.c,v 1.5 1993/11/24 00:49:13 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#ifndef lint
static char vcid[] = "$Id: u_scan.c,v 1.5 1993/11/24 00:49:13 doss Exp $";
#endif /* lint */

#include <mpi.h>
#include "coll.h"

/*@

MPI_User_scan - 

@*/
MPI_User_scan ( sendbuf, recvbuf, count, datatype, op, comm )
void             *sendbuf;
void             *recvbuf;
int               count;
MPI_Datatype      datatype;
void            (*op)();
MPI_Comm          comm;
{
  MPI_Status status;
  int        rank, size, extent;
  int        errno = MPI_SUCCESS;

  /* Check for invalid arguments */
  if ( (comm            == (MPI_Comm)0)   ||
       (comm->comm_type == MPIR_INTER)    ||
       (op              == (void (*)())0) ||
       (count           <= 0)             ||
       (sendbuf         == (void *)0)     ||
       (recvbuf         == (void *)0) )
    return (errno); 
  /* We also need to check that the datatype is a basic type? */

  /* Get my rank & size and switch communicators to the hidden collective */
  MPI_Comm_size ( comm, &size );
  MPI_Comm_rank ( comm, &rank );
  MPI_Type_extent ( datatype, &extent );
  comm = comm->comm_coll;

  /* If the recv and send buffers are equivalent */
  if (recvbuf == sendbuf) {
    if (rank > 0) {
      void *buffer = (void *)MALLOC(extent * count);
      MPI_Recv(buffer,count,datatype,rank-1,MPIR_USER_SCAN_TAG,comm,&status);
      (*op)(buffer, recvbuf, count, datatype); 
      FREE(buffer);
    }
  }
  else  /* send and recv buffers are different */
    if (rank > 0) {
      MPI_Recv(recvbuf,count,datatype,rank-1,MPIR_USER_SCAN_TAG,comm,&status);
      (*op)(sendbuf, recvbuf, count, datatype); 
    }
    else {
      MPI_Sendrecv(sendbuf,count,datatype,rank,
		   recvbuf,count,datatype,rank,
		   MPIR_USER_SCAN_TAG,comm, &status);
    }

  /* send the letter to destination */
  if (rank < (size-1)) 
    MPI_Send(recvbuf,count,datatype,rank+1,MPIR_USER_SCAN_TAG,comm);

  return(errno);
}
