/*
 *  $Id: ic_start.c,v 1.4 1993/11/12 12:31:42 doss Exp $
 *
 *  (C) 1993 by Argonne National Laboratory and Mississipi State University.
 *      All rights reserved.  See COPYRIGHT in top-level directory.
 */

#include <mpi.h>


/*@

MPI_Intercomm_start - create an intercommuncator from two intracommunicators

algorithm:
. 1) Allocate a "send_context"
. 2) Send context and lrank_to_grank list from local comm group 
     if I'm the local_leader.
@*/
int MPI_Intercomm_start ( local_comm, local_leader, peer_comm, 
                          remote_leader, tag, inter_request )
MPI_Comm       local_comm;
int                    local_leader;
MPI_Comm       peer_comm;
int                    remote_leader;
int                    tag;
MPI_Intercomm_request *inter_request;
{
  int context, *buffer, peer_rank, local_size, peer_size;

  /* Check for valid arguments to function */
  (void) MPI_Comm_size ( local_comm, &local_size );
  (void) MPI_Comm_size ( peer_comm,  &peer_size  );
  (void) MPI_Comm_rank ( peer_comm,  &peer_rank  );
  if ( (local_comm    == (MPI_Comm)0) ||
       (peer_comm     == (MPI_Comm)0) ||
       (local_leader  >= local_size) || 
       (local_leader  <  0) ||
       (remote_leader >= peer_size) || 
       (remote_leader <  0) ||
       (peer_rank     == MPI_UNDEFINED_RANK) )
    return (MPI_SUCCESS);
  
  /* Send the lrank_to_grank table of the local_comm and an allocated */
  /* context.  I may need to send a message saying how long the group */
  /* information is, then send this one along.  Actually, I'm ignorant */
  /* of how to do this using MPI pt2pt calls */
  MPIR_Context_alloc ( local_comm, 1, &context );

  /* Create MPI_Intercomm_request */
  (*inter_request) = (MPIR_Intercomm_request *) NEW (MPIR_Intercomm_request);
  /* Post isend (..... &(*inter_request)->shandle)*/
  /* Post irecv (..... &(*inter_request)->rhandle)*/
  (*inter_request)->tag = tag;
  (*inter_request)->local_comm = local_comm;
  (*inter_request)->peer_comm = peer_comm;
  (*inter_request)->local_leader = local_leader;
  (*inter_request)->remote_leader = remote_leader;
}

